==> smithy-0.2.0.tar.gz: lib/src/ast/pattern/uri_pattern.dart <==
// Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

import 'package:json_annotation/json_annotation.dart';
import 'package:smithy/ast.dart';

part 'uri_pattern.g.dart';

@JsonSerializable()
class UriPattern extends SmithyPattern {
  UriPattern({
    required String pattern,
    required List<Segment> segments,
    required this.queryLiterals,
  }) : super(pattern, segments);

  factory UriPattern.parse(String uri) {
    if (uri.endsWith('?')) {
      throw InvalidUriPatternError(
        'URI patterns must not end with "?". Found $uri',
      );
    } else if (!uri.startsWith('/')) {
      throw InvalidUriPatternError(
        'URI pattern must start with "/". Found $uri',
      );
    } else if (uri.contains('#')) {
      throw InvalidUriPatternError(
        'URI pattern must not contain a fragment. Found $uri',
      );
    }

    final parts = uri.split('?');
    final unparsedSegments = parts[0].split('/');
    final segments = <Segment>[];

    // Skip the first '/' segment, and thus assume offset of 1.
    for (int i = 1; i < unparsedSegments.length; i++) {
      String segment = unparsedSegments[i];
      if (i == unparsedSegments.length - 1 && segment.isEmpty) {
        break;
      }
      segments.add(Segment.parse(segment));
    }

    final Map<String, String> queryLiterals = {};
    // Parse the query literals outside of the general pattern
    if (parts.length == 2) {
      if (parts[1].contains('{') || parts[1].contains('}')) {
        throw InvalidUriPatternError(
          'URI labels must not appear in the query string. Found $uri',
        );
      }
      for (final kvp in parts[1].split('&')) {
        final parameterParts = kvp.split('=');
        String actualKey = parameterParts[0];
        if (queryLiterals.containsKey(actualKey)) {
          throw InvalidUriPatternError(
            'Literal query parameters must not be repeated: $uri',
          );
        }
        queryLiterals[actualKey] =
            parameterParts.length == 2 ? parameterParts[1] : '';
      }
    }

    return UriPattern(
      pattern: uri,
      segments: segments,
      queryLiterals: queryLiterals,
    );
  }

  factory UriPattern.fromJson(Map<String, Object?> json) =>
      _$UriPatternFromJson(json);

  final Map<String, String> queryLiterals;

  @override
  List<Object> get props => [
        queryLiterals,
        ...super.props,
      ];

  Map<String, Object?> toJson() => _$UriPatternToJson(this);
}

==> smithy-0.6.2.tar.gz: lib/src/ast/pattern/uri_pattern.dart <==
// Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
// SPDX-License-Identifier: Apache-2.0

import 'package:json_annotation/json_annotation.dart';
import 'package:smithy/ast.dart';

part 'uri_pattern.g.dart';

@JsonSerializable()
class UriPattern extends SmithyPattern {
  UriPattern({
    required String pattern,
    required List<Segment> segments,
    required this.queryLiterals,
  }) : super(pattern, segments);

  factory UriPattern.parse(String uri) {
    if (uri.endsWith('?')) {
      throw InvalidUriPatternError(
        'URI patterns must not end with "?". Found $uri',
      );
    } else if (!uri.startsWith('/')) {
      throw InvalidUriPatternError(
        'URI pattern must start with "/". Found $uri',
      );
    } else if (uri.contains('#')) {
      throw InvalidUriPatternError(
        'URI pattern must not contain a fragment. Found $uri',
      );
    }

    final parts = uri.split('?');
    final unparsedSegments = parts[0].split('/');
    final segments = <Segment>[];

    // Skip the first '/' segment, and thus assume offset of 1.
    for (var i = 1; i < unparsedSegments.length; i++) {
      final segment = unparsedSegments[i];
      if (i == unparsedSegments.length - 1 && segment.isEmpty) {
        break;
      }
      segments.add(Segment.parse(segment));
    }

    final queryLiterals = <String, String>{};
    // Parse the query literals outside of the general pattern
    if (parts.length == 2) {
      if (parts[1].contains('{') || parts[1].contains('}')) {
        throw InvalidUriPatternError(
          'URI labels must not appear in the query string. Found $uri',
        );
      }
      for (final kvp in parts[1].split('&')) {
        final parameterParts = kvp.split('=');
        final actualKey = parameterParts[0];
        if (queryLiterals.containsKey(actualKey)) {
          throw InvalidUriPatternError(
            'Literal query parameters must not be repeated: $uri',
          );
        }
        queryLiterals[actualKey] =
            parameterParts.length == 2 ? parameterParts[1] : '';
      }
    }

    return UriPattern(
      pattern: uri,
      segments: segments,
      queryLiterals: queryLiterals,
    );
  }

  factory UriPattern.fromJson(Map<String, Object?> json) =>
      _$UriPatternFromJson(json);

  final Map<String, String> queryLiterals;

  @override
  List<Object> get props => [
        queryLiterals,
        ...super.props,
      ];

  Map<String, Object?> toJson() => _$UriPatternToJson(this);
}
